import {
  Logger,
  getTimestamp
} from "./chunk-CYX5OLOK.js";

// src/common/orderby.ts
function parseOrderby(value = "") {
  let [field = "", orderby = "asc"] = value.split(" ");
  orderby = orderby.toLowerCase();
  return {
    field,
    orderby,
    asc: orderby !== "desc",
    desc: orderby === "desc"
  };
}
function composeOrderby(field, asc = true) {
  return `${field} ${asc ? "asc" : "desc"}`;
}
function cmp(a, b, asc = true) {
  const aa = a || 0;
  const bb = b || 0;
  return aa > bb ? asc ? 1 : -1 : aa < bb ? asc ? -1 : 1 : 0;
}
function sortedOrderby(values, orderby) {
  let { field, asc } = parseOrderby(orderby);
  if (field != null && typeof field === "string") {
    let sortValues = Array.from(values);
    sortValues.sort((a, b) => {
      return cmp(a[field], b[field], asc);
    });
    return sortValues;
  }
  return values;
}

// src/common/array.ts
function arrayUnique(x) {
  return x.filter((n, index) => x.indexOf(n) === index);
}
function arrayMinus(x, y) {
  return arrayUnique(x.filter((n) => !y.includes(n)));
}
function arrayUnion(...a) {
  return arrayUnique(a.reduce((acc = [], value) => acc.concat(value), []));
}
function arrayIntersection(x, y) {
  return arrayUnique(x).filter((n) => y.includes(n));
}
function arraySymmetricDifference(x, y) {
  return arrayMinus(arrayUnion(x, y), arrayIntersection(x, y));
}
function arrayRemoveElement(arr, el) {
  if (arr && Array.isArray(arr)) {
    let index;
    while ((index = arr.indexOf(el)) !== -1) {
      arr.splice(index, 1);
    }
    return arr;
  }
  return [];
}
function arraySetElement(arr, el) {
  if (!arr.includes(el))
    arr.push(el);
  return arr;
}
function arrayFilterInPlace(array, fn) {
  array.splice(0, array.length, ...array.filter(fn));
  return array;
}
function arrayToggleInPlace(array, el) {
  const index = array.findIndex((e) => e === el);
  if (index >= 0)
    array.splice(index, 1);
  else
    array.push(el);
  return array;
}
function arrayEmptyInPlace(array) {
  array.splice(0, array.length);
  return array;
}
function arraySorted(arr, cond = cmp) {
  let copy = Array.from(arr);
  copy.sort(cond);
  return copy;
}
function arraySortedNumbers(arr) {
  return arraySorted(arr, (l, r) => l - r);
}
function arrayIsEqual(array1, array2) {
  return array1.length === array2.length && array1.every((value, index) => value === array2[index]);
}
function arrayShuffleInPlace(array) {
  array.sort(() => Math.random() > 0.5 ? 1 : -1);
  return array;
}
function arrayShuffle(array) {
  return arrayShuffleInPlace(Array.from(array));
}
function arrayShuffleForce(array) {
  while (array.length > 1) {
    const copy = Array.from(array);
    arrayShuffleInPlace(copy);
    if (!arrayIsEqual(array, copy))
      return copy;
  }
  return array;
}
function arrayRandomElement(array) {
  return array[Math.floor(Math.random() * array.length)];
}

// src/common/basex.ts
var log = Logger("basex");
var alphabets = {
  "2": "01",
  "8": "01234567",
  "11": "0123456789a",
  "16": "0123456789abcdef",
  "32": "0123456789ABCDEFGHJKMNPQRSTVWXYZ",
  "36": "0123456789abcdefghijklmnopqrstuvwxyz",
  "58": "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz",
  "62": "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",
  "64": "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",
  "66": "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_.!~"
};
function useBase(alphaOrBase) {
  let ALPHABET;
  if (typeof alphaOrBase === "string") {
    ALPHABET = alphaOrBase;
  } else {
    ALPHABET = alphabets[alphaOrBase.toString()];
    if (ALPHABET == null)
      throw new Error(`Unknown base ${alphaOrBase}`);
  }
  if (ALPHABET.length >= 255)
    throw new TypeError("Alphabet too long");
  const BASE_MAP = new Uint8Array(256);
  for (let j = 0; j < BASE_MAP.length; j++) {
    BASE_MAP[j] = 255;
  }
  for (let i = 0; i < ALPHABET.length; i++) {
    const x = ALPHABET.charAt(i);
    const xc = x.charCodeAt(0);
    if (BASE_MAP[xc] !== 255)
      throw new TypeError(x + " is ambiguous");
    BASE_MAP[xc] = i;
  }
  const BASE = ALPHABET.length;
  const LEADER = ALPHABET.charAt(0);
  const FACTOR = Math.log(BASE) / Math.log(256);
  const iFACTOR = Math.log(256) / Math.log(BASE);
  function encode(source, padToLength = -1) {
    let data;
    if (source instanceof ArrayBuffer) {
      data = new Uint8Array(source);
    } else {
      data = source;
    }
    if (data.length === 0)
      return "";
    let length = 0;
    let pbegin = 0;
    const pend = data.length;
    while (pbegin !== pend && data[pbegin] === 0)
      pbegin++;
    const size2 = (pend - pbegin) * iFACTOR + 1 >>> 0;
    const dataEncoded = new Uint8Array(size2);
    while (pbegin !== pend) {
      let carry = data[pbegin];
      let i = 0;
      for (let it1 = size2 - 1; (carry !== 0 || i < length) && it1 !== -1; it1--, i++) {
        carry += 256 * dataEncoded[it1] >>> 0;
        dataEncoded[it1] = carry % BASE >>> 0;
        carry = carry / BASE >>> 0;
      }
      if (carry !== 0) {
        log.warn("Non-zero carry", data, padToLength, i, size2);
        throw new Error("Non-zero carry");
      }
      length = i;
      pbegin++;
    }
    let it2 = size2 - length;
    while (it2 !== size2 && dataEncoded[it2] === 0) {
      it2++;
    }
    let str = "";
    for (; it2 < size2; ++it2)
      str += ALPHABET.charAt(dataEncoded[it2]);
    if (padToLength > 0) {
      return str.padStart(padToLength, LEADER);
    }
    return str;
  }
  function decode(source, padToLength = -1) {
    if (typeof source !== "string")
      throw new TypeError("Expected String");
    if (source.length === 0)
      return new Uint8Array();
    source = source.replaceAll(/\s+/gi, "");
    let psz = 0;
    let length = 0;
    while (source[psz] === LEADER) {
      psz++;
    }
    const size2 = (source.length - psz) * FACTOR + 1 >>> 0;
    const dataDecoded = new Uint8Array(size2);
    while (source[psz]) {
      let carry = BASE_MAP[source.charCodeAt(psz)];
      if (carry === 255)
        throw new Error(`Unsupported character "${source[psz]}"`);
      let i = 0;
      for (let it3 = size2 - 1; (carry !== 0 || i < length) && it3 !== -1; it3--, i++) {
        carry += BASE * dataDecoded[it3] >>> 0;
        dataDecoded[it3] = carry % 256 >>> 0;
        carry = carry / 256 >>> 0;
      }
      if (carry !== 0)
        throw new Error("Non-zero carry");
      length = i;
      psz++;
    }
    let it4 = size2 - length;
    while (it4 !== size2 && dataDecoded[it4] === 0) {
      it4++;
    }
    if (padToLength > 0) {
      return new Uint8Array([
        ...new Uint8Array(padToLength - dataDecoded.length + it4),
        ...dataDecoded.slice(it4)
      ]);
    }
    return dataDecoded.slice(it4);
  }
  return {
    encode,
    decode
  };
}
var { encode: encodeBase16, decode: decodeBase16 } = useBase(16);
var { encode: encodeBase32, decode: decodeBase32 } = useBase(32);
var { encode: encodeBase58, decode: decodeBase58 } = useBase(58);
var { encode: encodeBase62, decode: decodeBase62 } = useBase(62);

// src/common/camelcase.ts
var toCamelCase = (s) => {
  if (s.length > 0) {
    if (/^[A-Z0-9_\-\ ]*$/g.test(s)) {
      s = s.toLowerCase();
    }
    s = s.replace(/^[-_\ ]+/gi, "").replace(/[-_\ ]+$/gi, "").replace(/[-_\ ]+([a-z0-9])/gi, ($0, $1) => $1.toUpperCase());
    s = s[0].toLowerCase() + s.substring(1);
  }
  return s;
};

// src/common/convert.ts
function stringToBoolean(value, defaultValue = false) {
  if (value == null || typeof value !== "string")
    return defaultValue;
  return ["1", "true", "yes"].includes(value.toLocaleLowerCase());
}
function stringToInteger(value, defaultValue = 0) {
  var _a;
  if (value == null || typeof value !== "string")
    return defaultValue;
  return (_a = parseInt(value.trim(), 10)) != null ? _a : defaultValue;
}
function stringToFloat(value, defaultValue = 0) {
  var _a;
  if (value == null || typeof value !== "string")
    return defaultValue;
  return (_a = parseFloat(value.trim())) != null ? _a : defaultValue;
}
function valueToBoolean(value, defaultValue = false) {
  if (value == null)
    return defaultValue;
  if (typeof value === "boolean")
    return value;
  if (typeof value === "number")
    return value !== 0;
  return ["1", "true", "yes"].includes(value.toString().toLocaleLowerCase());
}
function valueToInteger(value, defaultValue = 0) {
  var _a;
  if (value == null)
    return defaultValue;
  if (typeof value === "boolean")
    return value ? 1 : 0;
  if (typeof value === "number")
    return Math.floor(value);
  return (_a = parseInt(value.toString().trim(), 10)) != null ? _a : defaultValue;
}
function valueToFloat(value, defaultValue = 0) {
  var _a;
  if (value == null)
    return defaultValue;
  if (typeof value === "boolean")
    return value ? 1 : 0;
  if (typeof value === "number")
    return Math.floor(value);
  return (_a = parseFloat(value.toString().trim())) != null ? _a : defaultValue;
}
function valueToString(value, defaultValue = "") {
  var _a;
  if (value == null)
    return defaultValue;
  return (_a = value.toString()) != null ? _a : defaultValue;
}
var toFloat = valueToFloat;
var toInt = valueToInteger;
var toString = valueToString;
var toBool = valueToBoolean;
function serializer(replacer, cycleReplacer) {
  var stack = [], keys = [];
  if (cycleReplacer == null)
    cycleReplacer = function(key, value) {
      if (stack[0] === value)
        return "[Circular ~]";
      return "[Circular ~." + keys.slice(0, stack.indexOf(value)).join(".") + "]";
    };
  return function(key, value) {
    if (stack.length > 0) {
      var thisPos = stack.indexOf(this);
      ~thisPos ? stack.splice(thisPos + 1) : stack.push(this);
      ~thisPos ? keys.splice(thisPos, Infinity, key) : keys.push(key);
      if (~stack.indexOf(value))
        value = cycleReplacer == null ? void 0 : cycleReplacer.call(this, key, value);
    } else
      stack.push(value);
    return replacer == null ? value : replacer.call(this, key, value);
  };
}
function stringify(obj, replacer, spaces, cycleReplacer) {
  return JSON.stringify(obj, serializer(replacer, cycleReplacer), spaces);
}
function formatMessages(messages2, opt = {}) {
  const { trace = true, pretty = true } = opt;
  return messages2.map((obj) => {
    if (obj && typeof obj === "object") {
      if (obj instanceof Error) {
        if (!trace) {
          return `${obj.name || "Error"}: ${obj.message}`;
        }
        return `${obj.name || "Error"}: ${obj.message}
${obj.stack}`;
      }
      return pretty ? stringify(obj, null, 2) : stringify(obj);
    }
    return String(obj);
  });
}
function renderMessages(messages2, opt = {}) {
  return formatMessages(messages2, opt).join(" ");
}

// src/common/csv.ts
var separator = ",";
var preventCast = false;
var ignoreNullOrUndefined = true;
function filterFloat(value) {
  if (/^([-+])?([0-9]+(\.[0-9]+)?|Infinity)$/.test(value)) {
    return Number(value);
  }
  return NaN;
}
function escape(field) {
  if (ignoreNullOrUndefined && field == void 0) {
    return "";
  }
  if (preventCast) {
    return '="' + String(field).replace(/"/g, '""') + '"';
  }
  if (!isNaN(filterFloat(field)) && isFinite(field)) {
    return parseFloat(field);
  }
  return '"' + String(field).replace(/"/g, '""') + '"';
}
function csv(data, headerRow) {
  let body = "";
  if (headerRow) {
    body = headerRow.join(separator) + "\r\n";
  }
  for (let i = 0; i < data.length; i++) {
    body += data[i].map(escape).join(separator) + "\r\n";
  }
  return body;
}

// src/common/promise.ts
var { warn } = Logger("zeed:promise");
async function sleep(milliSeconds) {
  return new Promise((resolve) => setTimeout(resolve, milliSeconds));
}
async function immediate() {
  return new Promise((resolve) => setTimeout(resolve, 0));
}
var timeoutReached = Symbol("timeout");
async function timeout(promise, milliSeconds, timeoutValue = timeoutReached) {
  return new Promise(async (resolve, reject) => {
    let done = false;
    const timeout2 = setTimeout(() => {
      done = true;
      resolve(timeoutValue);
    }, milliSeconds);
    try {
      let result = await promise;
      clearTimeout(timeout2);
      if (!done)
        resolve(result);
    } catch (err) {
      clearTimeout(timeout2);
      if (!done)
        reject(err);
    }
  });
}
var timoutError = new Error("Timeout reached");
function isTimeout(value) {
  return value === timeoutReached || value === timoutError;
}
async function tryTimeout(promise, milliSeconds) {
  return new Promise(async (resolve, reject) => {
    let done = false;
    const timeout2 = setTimeout(() => {
      done = true;
      reject(timoutError);
    }, milliSeconds);
    try {
      let result = await promise;
      clearTimeout(timeout2);
      if (!done)
        resolve(result);
    } catch (err) {
      clearTimeout(timeout2);
      if (!done)
        reject(err);
    }
  });
}
function waitOn(obj, event, timeoutMS = 1e3) {
  return new Promise((resolve, reject) => {
    let fn = (value) => {
      if (timer) {
        clearTimeout(timer);
        done();
        resolve(value);
      }
    };
    let done = () => {
      timer = null;
      if (obj.off) {
        obj.off(event, fn);
      } else if (obj.removeEventListener) {
        obj.removeEventListener(event, fn);
      } else {
        warn("No remove listener method found for", obj, event);
      }
    };
    let timer = setTimeout(() => {
      done();
      reject(new Error(`Did not response in time`));
    }, timeoutMS);
    if (obj.on) {
      obj.on(event, fn);
    } else if (obj.addEventListener) {
      obj.addEventListener(event, fn);
    } else {
      warn("No listener method found for", obj);
    }
  });
}
function isPromise(value) {
  return Boolean(value && (value instanceof Promise || typeof value.then === "function"));
}
function promisify(value) {
  return Promise.resolve(value);
}

// src/common/emitter.ts
var log2 = Logger("zeed:emitter");
var Emitter = class {
  constructor() {
    this.subscribers = {};
    this.subscribersOnAny = [];
  }
  async emit(event, ...args) {
    let ok = false;
    try {
      let subscribers = this.subscribers[event] || [];
      this.subscribersOnAny.forEach((fn) => fn(event, ...args));
      if (subscribers.length > 0) {
        let all = subscribers.map((fn) => {
          try {
            return promisify(fn(...args));
          } catch (err) {
            log2.warn("emit warning:", err);
          }
        });
        ok = true;
        await Promise.all(all);
      }
    } catch (err) {
      log2.error("emit exception", err);
    }
    return ok;
  }
  onAny(fn) {
    this.subscribersOnAny.push(fn);
  }
  on(event, listener) {
    let subscribers = this.subscribers[event] || [];
    subscribers.push(listener);
    this.subscribers[event] = subscribers;
    return {
      cleanup: () => {
        this.off(event, listener);
      }
    };
  }
  once(event, listener) {
    const onceListener = async (...args) => {
      this.off(event, onceListener);
      return await promisify(listener(...args));
    };
    this.on(event, onceListener);
  }
  off(event, listener) {
    this.subscribers[event] = (this.subscribers[event] || []).filter((f) => listener && f !== listener);
    return this;
  }
  removeAllListeners(event) {
    this.subscribers = {};
    return this;
  }
};
var messages = new Emitter();
function lazyListener(emitter, listenerKey) {
  const name = Math.round(Math.random() * 100);
  var events = [];
  var lazyResolve;
  const incoming = (key, obj) => {
    let ev = { key, obj };
    events.push(ev);
    lazyResolve && lazyResolve();
  };
  if (listenerKey) {
    if (emitter.on) {
      emitter.on(listenerKey, (obj) => {
        incoming(listenerKey, obj);
      });
    } else if (emitter.addEventListener) {
      emitter.addEventListener(listenerKey, (obj) => {
        incoming(listenerKey, obj);
      });
    } else {
      log2.error(name, "Cannot listen to key");
    }
  } else {
    if (emitter.onAny) {
      emitter.onAny((key, obj) => {
        incoming(key, obj);
      });
    } else {
      log2.error(name, "cannot listen to all for", emitter);
    }
  }
  let on = (key, skipUnmatched = true) => {
    return new Promise((resolve, reject) => {
      if (!key) {
        key = listenerKey;
        if (!key) {
          if (events.length) {
            key = events[0].key;
          }
        }
      }
      lazyResolve = () => {
        while (events.length > 0) {
          let ev = events.shift();
          if (ev.key === key) {
            lazyResolve = void 0;
            resolve(ev.obj);
          } else {
            if (skipUnmatched) {
              log2.warn(name, `Unhandled event ${key} with value: ${ev.obj}`);
              continue;
            }
            reject(`Expected ${key}, but found ${ev.key} with value=${ev.obj}`);
            log2.error(name, `Unhandled event ${key} with value: ${ev.obj}`);
          }
          break;
        }
      };
      lazyResolve();
    });
  };
  return on;
}

// src/common/mutex.ts
function createMutex() {
  let token = true;
  return (fn, elseFn) => {
    let executed = false;
    if (token) {
      token = false;
      try {
        fn();
        executed = true;
      } finally {
        token = true;
      }
    } else if (elseFn !== void 0) {
      elseFn();
    }
    return executed;
  };
}

// src/common/platform.ts
function getWindow() {
  if (typeof window !== "undefined")
    return window;
}
function getNavigator() {
  if (typeof navigator !== "undefined")
    return navigator;
}
function getGlobal() {
  var _a;
  return ((_a = getWindow()) != null ? _a : typeof WorkerGlobalScope !== "undefined") ? self : typeof global !== "undefined" ? global : Function("return this;")();
}
var _navigator = getNavigator();
var _window = getWindow();
function detect(info = {
  ios: false,
  macos: false,
  windows: false,
  beaker: false,
  electron: false,
  wkwebview: false,
  pwa: false,
  pwaInstalled: false,
  browser: false,
  node: false,
  worker: false,
  jest: false,
  macosNative: false,
  iosNative: false,
  appleNative: false,
  touch: false
}) {
  var _a, _b, _c, _d, _e, _f, _g, _h;
  info.ios = ((_a = _navigator == null ? void 0 : _navigator.platform) == null ? void 0 : _a.match(/(iPhone|iPod|iPad)/i)) != null;
  info.macos = !!((_b = _navigator == null ? void 0 : _navigator.platform) == null ? void 0 : _b.startsWith("Mac"));
  info.windows = !!((_c = _navigator == null ? void 0 : _navigator.platform) == null ? void 0 : _c.startsWith("Win"));
  info.beaker = (_window == null ? void 0 : _window["beaker"]) != null;
  info.electron = (((_e = (_d = _navigator == null ? void 0 : _navigator.userAgent) == null ? void 0 : _d.toLowerCase()) == null ? void 0 : _e.indexOf(" electron/")) || -1) > -1 && !info.beaker;
  info.wkwebview = ((_f = _window == null ? void 0 : _window.webkit) == null ? void 0 : _f["messageHandlers"]) != null;
  info.pwa = (_navigator == null ? void 0 : _navigator.serviceWorker) != null;
  info.pwaInstalled = (_navigator == null ? void 0 : _navigator.standalone) || ((_g = _window == null ? void 0 : _window.matchMedia("(display-mode: standalone)")) == null ? void 0 : _g.matches);
  info.node = typeof process !== "undefined" && ((_h = process == null ? void 0 : process.release) == null ? void 0 : _h.name) === "node";
  info.browser = !info.electron && !info.wkwebview && !info.node;
  info.worker = typeof WorkerGlobalScope !== "undefined" && self instanceof WorkerGlobalScope;
  info.jest = typeof jest !== "undefined";
  info.macosNative = info.wkwebview && info.macos;
  info.iosNative = info.wkwebview && info.ios;
  info.appleNative = info.wkwebview;
  info.touch = _window && "ontouchstart" in _window || ((_navigator == null ? void 0 : _navigator.maxTouchPoints) || 0) > 1 || (_navigator == null ? void 0 : _navigator.msPointerEnabled) && (_window == null ? void 0 : _window.MSGesture) || (_window == null ? void 0 : _window.DocumentTouch) && document instanceof DocumentTouch;
  return info;
}
var platform = detect();
function subscribeDetectOrientation(fn) {
  const mediaQuery = _window == null ? void 0 : _window.matchMedia("(orientation: landscape)");
  const emit = () => {
    const landscape = mediaQuery == null ? void 0 : mediaQuery.matches;
    fn({
      landscape,
      portrait: !landscape,
      orientation: landscape ? "landscape" : "portrait"
    });
  };
  emit();
  mediaQuery == null ? void 0 : mediaQuery.addEventListener("change", emit);
}

// src/common/sortable.ts
function startSortWeight(items) {
  return items.reduce((acc, item) => Math.min(acc, item.sort_weight || 0), 0) - 1 - Math.random();
}
function endSortWeight(items) {
  return items.reduce((acc, item) => Math.max(acc, item.sort_weight || 0), 0) + 1 + Math.random();
}
function moveSortWeight(newIndex, oldIndex, items) {
  let count = items.length;
  const moveLower = newIndex < oldIndex;
  if (count <= 0 || newIndex >= count - 1) {
    return endSortWeight(items);
  }
  if (newIndex <= 0) {
    return startSortWeight(items);
  }
  const step = moveLower ? -1 : 0;
  const lower = items[newIndex + step].sort_weight || 0;
  const upper = items[newIndex + step + 1].sort_weight || 0;
  const distance = upper - lower;
  if (distance === 0) {
    if (moveLower) {
      return startSortWeight(items);
    }
    return endSortWeight(items);
  }
  const middle = lower + distance / 2;
  const fuzzy = distance * 0.01 * (Math.random() - 0.5);
  return middle + fuzzy;
}
function sortedItems(items) {
  items.sort((a, b) => (a.sort_weight || 0) - (b.sort_weight || 0));
  return items;
}

// src/common/utils.ts
function size(obj) {
  if (obj != null) {
    if (obj.size != null) {
      return obj.size;
    }
    if (obj.length != null) {
      return obj.length;
    }
    return Object.keys(obj).length;
  }
  return 0;
}
function last(array) {
  return array != null && array.length > 0 ? array[array.length - 1] : void 0;
}
function empty(value) {
  try {
    if (value != null) {
      if (Array.isArray(value)) {
        return value.length <= 0;
      } else if (typeof value === "string") {
        return value.length <= 0;
      } else if ((value == null ? void 0 : value.size) != null) {
        return value.size <= 0;
      } else {
        return Object.keys(value).length <= 0;
      }
    }
  } catch (err) {
    console.error("Failed to check if empty for", value, err);
  }
  return true;
}
function cloneObject(obj) {
  return JSON.parse(JSON.stringify(obj));
}

// src/common/uuid.ts
var { encode: encode62, decode: decode62 } = useBase(62);
var { encode: encode32 } = useBase(32);
var _crypto = getGlobal().crypto || getGlobal()["msCrypto"];
function randomUint8Array(length = 16) {
  let rands = new Uint8Array(length);
  if (_crypto && _crypto["getRandomValues"]) {
    _crypto["getRandomValues"](rands);
  } else {
    for (let i = 0; i < length; i++) {
      rands[i] = Math.floor(Math.random() * 256);
    }
  }
  return rands;
}
function uuid() {
  return encode62(randomUint8Array(16), 22);
}
function uuidB32() {
  return encode32(randomUint8Array(16), 26);
}
var _unameCounters = {};
function uname(name = "id") {
  if (_unameCounters[name] == null) {
    _unameCounters[name] = 0;
  }
  return `${name}-${(_unameCounters[name]++).toString()}`;
}
var ReferenceDateInMS = 16e11;
function longToByteArray(long) {
  var byteArray = new Uint8Array([0, 0, 0, 0, 0, 0]);
  const bytes = byteArray.length - 1;
  for (var index = 0; index < byteArray.length; index++) {
    var byte = long & 255;
    byteArray[bytes - index] = byte;
    long = (long - byte) / 256;
  }
  return byteArray;
}
function suidBytes() {
  const ms = getTimestamp() - ReferenceDateInMS;
  return new Uint8Array([...longToByteArray(ms), ...randomUint8Array(10)]);
}
function suid() {
  return encode62(suidBytes(), 22);
}
function suidDate(id) {
  return suidBytesDate(decode62(id, 16));
}
function suidBytesDate(id) {
  return new Date(ReferenceDateInMS + id.slice(0, 6).reduce((acc, byte) => {
    return acc * 256 + byte;
  }, 0));
}

// src/common/queue.ts
var log3 = Logger("queue");
var SerialQueue = class {
  constructor() {
    this.queue = [];
    this.isPaused = false;
    this.waitToFinish = [];
  }
  async performNext() {
    if (this.currentTask != null) {
      return;
    }
    while (this.currentTask == null && !this.isPaused) {
      let info = this.queue.shift();
      if (info == null) {
        break;
      }
      const { task, resolve } = info;
      this.currentTask = task();
      let result = void 0;
      try {
        result = await this.currentTask;
      } catch (err) {
        log3.error("Error performing task", err);
      }
      resolve(result);
      this.currentTask = void 0;
    }
    while (this.waitToFinish.length > 0) {
      this.waitToFinish.shift()();
    }
  }
  async enqueue(task) {
    return new Promise((resolve) => {
      this.queue.push({ task, resolve });
      this.performNext();
    });
  }
  async cancelAll(unblock = true) {
    let resolver = this.queue.map((task) => task.resolve);
    this.queue = [];
    resolver.forEach((r) => r(void 0));
    await this.wait();
  }
  async pause() {
    this.isPaused = true;
    await this.wait();
  }
  resume() {
    this.isPaused = false;
    this.performNext();
  }
  async wait() {
    if (this.currentTask == null && (this.queue.length === 0 || this.isPaused)) {
      return;
    }
    return new Promise((resolve) => {
      this.waitToFinish.push(resolve);
    });
  }
};

// src/common/channel.ts
var log4 = Logger("channel");
var Channel = class extends Emitter {
  close() {
  }
};
var LocalChannel = class extends Channel {
  postMessage(data) {
    var _a;
    (_a = this.other) == null ? void 0 : _a.emit("message", {
      data: cloneObject(data),
      origin: "local",
      lastEventId: uuid()
    });
  }
};
var CommandChannel = class {
};

export {
  parseOrderby,
  composeOrderby,
  cmp,
  sortedOrderby,
  arrayUnique,
  arrayMinus,
  arrayUnion,
  arrayIntersection,
  arraySymmetricDifference,
  arrayRemoveElement,
  arraySetElement,
  arrayFilterInPlace,
  arrayToggleInPlace,
  arrayEmptyInPlace,
  arraySorted,
  arraySortedNumbers,
  arrayIsEqual,
  arrayShuffleInPlace,
  arrayShuffle,
  arrayShuffleForce,
  arrayRandomElement,
  useBase,
  encodeBase16,
  decodeBase16,
  encodeBase32,
  decodeBase32,
  encodeBase58,
  decodeBase58,
  encodeBase62,
  decodeBase62,
  toCamelCase,
  stringToBoolean,
  stringToInteger,
  stringToFloat,
  valueToBoolean,
  valueToInteger,
  valueToFloat,
  valueToString,
  toFloat,
  toInt,
  toString,
  toBool,
  stringify,
  formatMessages,
  renderMessages,
  csv,
  sleep,
  immediate,
  timeoutReached,
  timeout,
  timoutError,
  isTimeout,
  tryTimeout,
  waitOn,
  isPromise,
  promisify,
  Emitter,
  messages,
  lazyListener,
  createMutex,
  getWindow,
  getNavigator,
  getGlobal,
  detect,
  platform,
  subscribeDetectOrientation,
  startSortWeight,
  endSortWeight,
  moveSortWeight,
  sortedItems,
  size,
  last,
  empty,
  cloneObject,
  uuid,
  uuidB32,
  uname,
  suidBytes,
  suid,
  suidDate,
  suidBytesDate,
  SerialQueue,
  Channel,
  LocalChannel,
  CommandChannel
};
//# sourceMappingURL=chunk-LZJPMHAM.js.map
